/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.util.Arrays;
import javax.annotation.Nullable;
import net.java.truelicense.core.io.*;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * Defines parameters for accessing a {@link java.security.KeyStore} which
 * holds the public or private keys to sign or verify a {@link RepositoryModel}
 * by an {@link Authentication}.
 * <p>
 * Unless stated otherwise, all no-argument methods need to return consistent
 * objects so that caching them is not required.
 * A returned object is considered to be consistent if it compares
 * {@linkplain Object#equals(Object) equal} or at least behaves identical to
 * any previously returned object.
 *
 * @author Christian Schlichtherle
 */
public interface KeyStoreParameters
extends AuthenticationParameters, SourceProvider {

    /**
     * Returns the nullable input source for the key store.
     *
     * @return the nullable input source for the key store.
     *         May be {@code null} if and only if the key store type does not
     *         require loading from an input stream.
     */
    @Override @Nullable Source source();

    /**
     * Returns the type of the key store, e.g. {@code "JKS"}.
     * The returned string should be computed on demand from an obfuscated form,
     * e.g. by annotating a constant string value with the \@{@link Obfuscate}
     * annotation and processing it with the TrueLicense Maven Plugin.
     */
    String storeType();

    /**
     * Returns a new char array with the password for the key store.
     * It is the caller's responsibility to wipe the contents of the char array
     * after use, e.g. by a call to {@link Arrays#fill(char[], char)}.
     *
     * @return A new char array with the password for the key store.
     */
    char[] storePassword();

    /** Returns the alias of the entry in the key store. */
    String alias();

    /**
     * Returns a nullable or new char array with the password for the private
     * key in the entry.
     * It is the caller's responsibility to wipe the contents of the char array
     * after use, e.g. by a call to {@link Arrays#fill(char[], char)}.
     * <p>
     * The returned array must be non-{@code null} if and only if the entry
     * in the key store is a private key entry (as opposed to a trusted
     * certificate entry or a secret key entry).
     * A private key entry is only required to create license keys.
     * When installing or verifying license keys, you should never use a
     * private key entry, but only a trusted certificate entry.
     * Otherwise, you'll put the security of your licensing scheme at risk!
     *
     * @return A nullable or new char array with the password for the private
     *         key in the entry.
     */
    @Nullable char[] keyPassword();
}
