/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.nio.charset.Charset;
import java.security.*;
import javax.annotation.*;
import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.codec.*;
import net.java.truelicense.core.io.*;
import org.apache.commons.codec.binary.Base64;

/**
 * A controller for signing and verifying objects (artifacts) in a
 * {@link RepositoryModel} with the help of a {@link Codec}.
 * This class is used by {@linkplain Notary notaries} to
 * {@linkplain Authentication#sign sign} or {@linkplain Authentication#verify verify} artifacts.
 *
 * @author Christian Schlichtherle
 */
@Immutable
abstract class RepositoryController {

    private final Base64 base64 = new Base64();

    /** Returns the codec for the artifact. */
    abstract Codec codec();

    private @Nullable byte[] data(
            final @CheckForNull String body,
            final @CheckForNull Charset charset) {
        if (null == body) return null;
        return null == charset ? decode(body) : body.getBytes(charset);
    }

    private @Nullable byte[] decode(final @CheckForNull String body) {
        return base64.decode(body);
    }

    private @Nullable String body(
            final @CheckForNull byte[] artifact,
            final @CheckForNull Charset charset) {
        if (null == artifact) return null;
        return null == charset ? encode(artifact) : new String(artifact, charset);
    }

    private @Nullable String encode(final @CheckForNull byte[] data) {
        return base64.encodeToString(data);
    }

    /**
     * Encodes and signs the given {@code artifact}
     * and returns an artifactory for it.
     *
     * @param  repository the repository for encoding the artifact.
     * @param  engine the signature engine.
     * @param  key the private key.
     * @param  artifact the nullable artifact.
     */
    <A> Artifactory<A> sign(
            final Repository repository,
            final Signature engine,
            final PrivateKey key,
            final @CheckForNull A artifact)
    throws Exception {
        final RepositoryModel model = repository.model();
        final Codec codec = codec();
        final Charset charset = Codecs.charset(codec);
        final MemoryStore store = new MemoryStore();
        codec.encode(store, artifact);
        final byte[] artifactData = store.data();
        engine.initSign(key);
        engine.update(artifactData);
        final byte[] signatureData = engine.sign();
        model.setArtifact(body(artifactData, charset));
        model.setSignature(encode(signatureData));
        model.setAlgorithm(engine.getAlgorithm());
        return new EncodedArtifact<A>(codec, store);
    }

    /**
     * Verifies the signature of the encoded artifact
     * and returns an artifactory for it.
     *
     * @param  repository the repository for decoding the artifact.
     * @param  engine the signature engine.
     * @param  key the public key.
     * @throws IllegalStateException if the state of the model doesn't
     *         match the configuration of the codec or the signature engine.
     * @throws RepositoryIntegrityException if the integrity of this repositoryProvider
     *         with its encoded artifact has been compromised.
     */
    Artifactory<?> verify(
            final Repository repository,
            final Signature engine,
            final PublicKey key)
    throws Exception {
        final RepositoryModel model = repository.model();
        final Codec codec = codec();
        final Charset charset = Codecs.charset(codec);
        if (!model.getAlgorithm().equalsIgnoreCase(engine.getAlgorithm()))
            throw new IllegalStateException();
        engine.initVerify(key);
        final byte[] artifactData = data(model.getArtifact(), charset);
        engine.update(artifactData);
        if (!engine.verify(decode(model.getSignature())))
            throw new RepositoryIntegrityException();
        return new EncodedArtifact<Object>(codec, artifactData);
    }
}
