package net.java.truelicense.core.codec;

import java.nio.charset.Charset;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.annotation.CheckForNull;
import net.java.truelicense.obfuscate.Obfuscate;
import org.apache.commons.codec.Charsets;

/**
 * Utilities for codecs.
 *
 * @author Christian Schlichtherle
 */
public class Codecs {

    @Obfuscate private static final String _7BIT = "7bit";
    @Obfuscate private static final String _8BIT = "8bit";
    @Obfuscate private static final String QUOTED_PRINTABLE = "quoted-printable";
    @Obfuscate private static final String BASE64 = "base64";

    @Obfuscate private static final String CHARSET_PATTERN =
            "\\s*charset\\s*=\\s*(?:\"([^\"]*)\"|([^\\s\\(\\)<>@,;:\\\\\"/\\[\\]\\?=]+))";

    /** Not perfect, but should work for valid input. */
    private static final Pattern pattern =
            Pattern.compile(CHARSET_PATTERN, Pattern.CASE_INSENSITIVE);

    /**
     * Figures the character set which is used by the given codec.
     * <ol>
     * <li>
     * First, the codec's
     * {@linkplain Codec#contentTransferEncoding Content-Transfer-Encoding}
     * gets tested:
     * If it is {@code "7bit"}, {@code "quoted-printable"} or {@code "base64"}
     * (ignoring case), then the returned character set is US-ASCII.
     * <li>
     * Otherwise, if the codec's Content-Transfer-Encoding is {@code "8bit"}
     * (ignoring case), then the codec's {@link Codec#contentType Content-Type}
     * gets tested:
     * <ol>
     * <li>
     * If the codec's Content-Type specifies a {@code charset} parameter
     * (ignoring case), then an attempt is made to load this charset.
     * Upon success, this charset gets returned.
     * <li>
     * If the codec's Content-Type does not specify a {@code charset} parameter,
     * then UTF-8 gets assumed and returned for interoperability with JSON.
     * </ol>
     * <li>
     * Finally, in all other cases, {@code null} gets returned in order to
     * specify that the codec either uses an unknown character set or does not
     * produce text output.
     * </ol>
     *
     * @param  codec the codec to test.
     * @return The character set which is used by the given codec or
     *         {@code null} if the codec uses an unknown character set or does
     *         not produce text output.
     */
    public static @CheckForNull Charset charset(final Codec codec) {
        final String cte = codec.contentTransferEncoding();
        if (    _7BIT.equalsIgnoreCase(cte) ||
                QUOTED_PRINTABLE.equalsIgnoreCase(cte) ||
                BASE64.equalsIgnoreCase(cte)) {
            return Charsets.US_ASCII;
        } else  if (_8BIT.equalsIgnoreCase(cte)) {
            final Matcher m = pattern.matcher(codec.contentType());
            if (m.find()) {
                assert 2 == m.groupCount();
                String n = m.group(1);
                if (null == n) n = m.group(2);
                try { return Charset.forName(n); }
                catch (final RuntimeException fallThrough) { }
            } else {
                return Charsets.UTF_8;
            }
        }
        return null;
    }

    private Codecs() { }
}
